"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const ts = require("typescript");
function transformer(program) {
    return (context) => (file) => visitNodeAndChildren(file, program, context);
}
exports.default = transformer;
function visitNodeAndChildren(node, program, context) {
    // tslint:disable-next-line:max-line-length
    return ts.visitEachChild(visitNode(node, program), (childNode) => visitNodeAndChildren(childNode, program, context), context);
}
function visitNode(node, program) {
    const typeChecker = program.getTypeChecker();
    if (!isKeysCallExpression(node, typeChecker)) {
        return node;
    }
    if (!node.typeArguments) {
        return ts.createObjectLiteral();
    }
    const type = typeChecker.getTypeFromTypeNode(node.typeArguments[0]);
    const properties = typeChecker.getPropertiesOfType(type).filter((property) => property.declarations.length);
    const propertiesLiterals = properties.map((property) => {
        let property_type = typeChecker.typeToString(typeChecker.getTypeOfSymbolAtLocation(property, property.declarations[0]));
        if (property_type.includes("[]")) {
            property_type = "array";
        }
        return ts.createPropertyAssignment(property.name, ts.createLiteral(property_type));
    });
    return ts.createObjectLiteral(propertiesLiterals);
}
const indexTs = path.join(__dirname, "index.ts");
function isKeysCallExpression(node, typeChecker) {
    if (!ts.isCallExpression(node)) {
        return false;
    }
    const signature = typeChecker.getResolvedSignature(node);
    if (typeof signature === "undefined") {
        return false;
    }
    const { declaration } = signature;
    return !!declaration
        && !ts.isJSDocSignature(declaration)
        && (path.join(declaration.getSourceFile().fileName) === indexTs)
        && !!declaration.name
        && declaration.name.getText() === "schema";
}
